import React, {Component} from 'react'
import PropTypes from 'prop-types'
import {connect} from 'react-redux'
import { is, fromJS } from 'immutable'
import {Tabs, Icon} from 'antd'
import {modifyTabview, toggleIsiframe} from '@/store/action'
import asyncComponent from '@/utils/asyncComponent'
import NotFount from '@/components/404'
import './index.scss'
let Comps = {}

class Header extends Component {
  static propTpyes = {
    tabviews: PropTypes.array // 标签页数组
  }

  state = {
    selectedTabId: '', // 当前选中tab页面
    iFrameHeight: 0
  }

  handleTabview (menu) {
    // 关闭tab页，重新选择显示页
    let tabs = JSON.parse(JSON.stringify(this.props.tabviews))
    tabs = tabs.filter(tab => {
      if (tab.MenuID === this.state.selectedTabId) {
        tab.selected = true
      } else {
        tab.selected = false
      }
      return tab.MenuID !== menu.MenuID
    })
    if (menu.MenuID === this.state.selectedTabId) {
      tabs[0] && (tabs[0].selected = true)
    }
    this.props.modifyTabview(tabs)
  }

  changeTab (menu) {
    // 窗口切换
    this.setState({
      selectedTabId: menu.MenuID
    })

    this.resetWindow(menu)
  }

  selectcomponent (view) {
    // 根据tab页中菜单信息，选择所需的组件
    if (view.Remark === 'CommonTable') {
      return (<Comps.CommonTable MenuNo={view.MenuNo} key={view.MenuID}/>)
    } else if (view.LinkUrl.split('?')[0] === 'Main/Index') {
      return (<Comps.Iframe key={view.MenuID} title={view.MenuName} url={'http://qingqiumarket.cn/MKWMS/zh-CN/' + view.LinkUrl}/>)
    } else {
      return (<NotFount key={view.MenuID} />)
    }
  }

  resetWindow (view) {
    // 窗口在iframe与普通页面切换时，修改左侧菜单栏样式
    if (!view) return
    let _isiframe = this.props.isiframe
    if (view && view.Remark === 'CommonTable') {
      _isiframe = false
    } else if (view && view.LinkUrl.split('?')[0] === 'Main/Index') {
      _isiframe = true
    } else {
      _isiframe = false
    }

    if (_isiframe !== this.props.isiframe) {
      this.props.toggleIsiframe(_isiframe)
    }
  }

  UNSAFE_componentWillReceiveProps (nextProps) {
    if (nextProps.tabviews && !is(fromJS(this.props.tabviews), fromJS(nextProps.tabviews))) {
      // tab窗口页增加或删除
      if (nextProps.tabviews.length > this.props.tabviews.length) {
        // 查看新tab页需要组件是否加载
        let newtab = nextProps.tabviews[nextProps.tabviews.length - 1]
        if (!Comps.CommonTable && newtab.Remark === 'CommonTable') {
          Comps.CommonTable = asyncComponent(() => import('@/tabviews/commontable'))
        } else if (!Comps.Iframe && newtab.LinkUrl.split('?')[0] === 'Main/Index') {
          Comps.Iframe = asyncComponent(() => import('@/tabviews/iframe'))
        }
      }

      // 设置选中窗口
      let view = nextProps.tabviews.filter(tab => tab.selected)[0]
      this.setState({
        selectedTabId: view ? view.MenuID : ''
      })

      this.resetWindow(view)
    }
  }

  shouldComponentUpdate(nextProps, nextState) {
    return !is(fromJS(this.props), fromJS(nextProps)) || !is(fromJS(this.state), fromJS(nextState))
  }

  render () {
    return (
      <section className="flex-container content-box">
        <div className="content-header">
          {this.props.tabviews && this.props.tabviews.length > 0 &&
            <Tabs activeKey={this.state.selectedTabId}>
              {this.props.tabviews.map(view => {
                return (
                  <Tabs.TabPane
                    className="test"
                    tab={
                      <span>
                        <span className="tab-name" onClick={() => {this.changeTab(view)}}>
                          {view.MenuName}
                        </span>
                        <Icon type="close" onClick={() => {this.handleTabview(view)}}/>
                      </span>
                    }
                    key={view.MenuID}
                  >
                    {this.selectcomponent(view)}
                  </Tabs.TabPane>
                )
              })}
            </Tabs>
          }
        </div>
      </section>
    )
  }
}

const mapStateToProps = (state) => {
  return {
    tabviews: state.tabviews,
    isiframe: state.isiframe
  }
}

const mapDispatchToProps = (dispatch) => {
  return {
    modifyTabview: (tabviews) => dispatch(modifyTabview(tabviews)),
    toggleIsiframe: (isiframe) => dispatch(toggleIsiframe(isiframe))
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(Header)